#include "project.h"
#include "stdio.h"

/**
 * According to the Datasheet of Em_EEPROM 
 * linker script file
 * \<em_eepromproject_name>.cydsn\Generated+Source\PSoC(4)(5)\cm3gcc.ld
 * needs to be copied to a custom linker script, like custom_cm3gcc.ld
 * and in the custom_cm3gcc.ld following lines must be added after 
 * the line of
    .cy_checksum_exclude : { KEEP(*(.cy_checksum_exclude)) } >rom
 *
     EM_EEPROM_START_ADDRESS = <EEPROM Address>;
    .my_emulated_eeprom EM_EEPROM_START_ADDRESS :
    {
        KEEP(*(.my_emulated_eeprom))
    } >rom   
 * where, <EEPROM Address> is the start address of EEPROM part in the
 * flash rom. As CY8C5888LTI-LP97, I assined 0x30000 (which is 192KB point).
 * Then from Build Settings > Linker > General
 *   Custom Linker Script: .\Generated_Source\PSoC5\custom_cm3gcc.ld
 */

const uint8 store[Em_EEPROM_PHYSICAL_SIZE] 
CY_SECTION(".my_emulated_eeprom") __ALIGNED(CY_FLASH_SIZEOF_ROW) = { 0u } ;

cy_en_em_eeprom_status_t status ;

char str[128] ; /* print buffer */
void print(char *str)
{
    UART_PutString(str) ;
}

void print_em_eprom_error(cy_en_em_eeprom_status_t status)
{
    switch(status) {
    case CY_EM_EEPROM_SUCCESS:      print("No Error\n") ; break ;
    case CY_EM_EEPROM_BAD_PARAM:    print("The input parameter is invalid\n") ; break ;
    case CY_EM_EEPROM_BAD_CHECKSUM: print("The data in EEPROM is correupted\n") ; break ;
    case CY_EM_EEPROM_BAD_DATA:     print("Failed to place the EEPROM in flash\n") ; break ;
    case CY_EM_EEPROM_WRITE_FAIL:   print("Write to EEPROM failed\n") ; break ;
    default:
        sprintf(str, "EEPROM Unknown Error %d\n", status) ; 
        print(str) ;
        break ;
    }
}

void init_hardware(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */

    UART_Start() ; 
    
    status = Em_EEPROM_Init((uint32_t)store) ;
    if (status != CY_EM_EEPROM_SUCCESS) {
        print_em_eprom_error(status) ;
    }
}

void splash(void)
{
    sprintf(str, "Emulated EEPROM Test (%s %s)\n", __DATE__, __TIME__) ;
    print(str) ;
}

int main(void)
{
    unsigned int i ;
    unsigned int test_size = 16 ;
    
    uint8 zustand[EEPROM_SIZE] = {} ;
    init_hardware() ;
    
    splash() ;
    
    sprintf(str, "EEPROM SIZE = %d\n", EEPROM_SIZE) ;
    print(str) ;
    
    print("Store:\n") ;
    for (i = 0 ; i < test_size * 2 ; i++ ) {
        sprintf(str, "%02X ", store[i]) ;
        print(str) ;
        if (i == 0xF) {
            print("\n") ;
        }
    }
    print("\n") ;

    print("Values to be written:\n") ;
    for (i = 0 ; i < test_size ; i++) {
        zustand[i] = 0xF0 + i ;
        sprintf(str, "%02X ", zustand[i]) ;
        print(str) ;
    }
    print("\n") ;
    
    status = Em_EEPROM_Write((uint32)0, (void*)zustand, test_size) ;
    if (status != CY_EM_EEPROM_SUCCESS) {
        print_em_eprom_error(status) ;
    }
    
    print("Store:\n") ;
    for (i = 0 ; i < test_size * 2 ; i++ ) {
        sprintf(str, "%02X ", store[i]) ;
        print(str) ;
        if (i == 0xF) {
            print("\n") ;
        }
    }
    print("\n") ;
        
    for (i = 0 ; i < test_size ; i++ ) {
        zustand[i] = 0 ;
    }
    
    status = Em_EEPROM_Read((uint32)0, zustand, test_size) ;
    if (status != CY_EM_EEPROM_SUCCESS) {
        print_em_eprom_error(status) ;
    }
    
    print("Values read:\n") ;
    for (i = 0 ; i < test_size ; i++ ) {
        sprintf(str, "%02X ", zustand[i]) ;
        print(str) ;
    }
    print("\n") ;

    for(;;)
    {
        /* Place your application code here. */
    }
}

/* [] END OF FILE */
